!#################################################################
!
! TM5 save files
!
!### macros #####################################################
!
#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
!
#include "tm5.inc"

!
!#################################################################


module cy2_enkf_save

  use GO, only : gol, goErr, goPr
  
  implicit none
  
  
  ! --- in/out ----------------------------------
  
  private
  
  public  ::  ReadTracer_saved
  
  
  ! --- const --------------------------------
    
  character(len=*), parameter  ::  mname = 'savefiles'

  

contains

  subroutine ReadTracer_saved( region, fname, status)

    use file_hdf   , only : THdfFile, TSds, Init, Done, ReadAttribute, ReadData
    use grid       , only : TllGridInfo, TLevelInfo, Init, Done, Fill3D
    use dims       , only : im, jm, lm
    use chem_param , only : ntrace, ntracet, fscale, names, ntrace_chem
    use partools   , only : myid, root
    use partools   , only : previous_par, tracer_active, ntracetloc, tracer_loc
    use partools   , only : Par_Barrier
    use meteodata  , only : lli, levi, sp_dat,m_dat
    use tracer_data, only : mass_dat

    ! --- in/out -------------------------------
    
    integer, intent(in)           ::  region
    character(len=*), intent(in)  ::  fname
    integer, intent(out)          ::  status
    
    ! --- const -------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/ReadTracer_saved'
    

    ! --- local -------------------------------
    
    integer                             ::  imr, jmr, lmr

    real, dimension(:,:,:), pointer     ::  sp
    real, dimension(:,:,:), pointer     ::  m
    real, dimension(:,:,:,:), pointer   ::  rm

    type(THdfFile)                   ::  hdf
    type(TSds)                       ::  sds_rm, sds_rmc
    integer                          ::  n, n_loc
    integer                          ::  ntrace_in, ntracet_in
    integer                          ::  n_in, nl
    character(len=8), allocatable    ::  names_in(:)
    integer                          ::  xbeg_in, ybeg_in
    real                             ::  dx_in, dy_in
    integer                          ::  im_in, jm_in, lm_in
    real, allocatable                ::  at_in(:), bt_in(:)
    type(TllGridInfo)                ::  lli_in
    type(TLevelInfo)                 ::  levi_in
    real, allocatable                ::  rm_in(:,:,:,:)
    real, allocatable                ::  rmt(:,:,:)
    integer                          ::  l
    character(len=40)                :: xx

    ! --- begin -------------------------------
    
    ! should be parallel over tracers ...
    if ( previous_par(region) /= 'tracer' ) then
      write (gol,'("expected parallel over tracers:")'); call goErr
      write (gol,'("  region       : ",i3)') region; call goErr
      write (gol,'("  previous_par : ",a)') trim(previous_par(region)); call goErr
      write (gol,'("in ",a)') rname; call goErr; status=1; return
    end if
    
    ! grid sizes
    imr = im(region)
    jmr = jm(region)
    lmr = lm(region)
    
    ! set pointers
    sp => sp_dat(region)%data
    m  => m_dat(region)%data
    rm => mass_dat(region)%rm_t

    ! open hdf file (all pe)
    write(xx,*) myid
    xx=adjustl(xx)
    call Init( hdf, trim(fname)//'_pe'//trim(xx), 'read', status )
    IF_NOTOK_RETURN(status=1)

    ! read list with tracer names in file:
    call ReadAttribute( hdf, 'ntrace', ntrace_in, status )
    IF_NOTOK_RETURN(status=1)
    call ReadAttribute( hdf, 'ntracet', ntracet_in, status )
    IF_NOTOK_RETURN(status=1)
    IF (ntracet_in.ne.ntracet) THEN
      write (gol,'("ntracet does not correspond to ntracet in save file:")'); call goErr
      write (gol,'("  ntracet      : ",i3)') ntracet; call goErr
      write (gol,'("  ntracet_in   : ",i3)') ntracet_in; call goErr
      write (gol,'("Routine requires them to be equal")'); call goErr
      write (gol,'("in ",a)') rname; call goErr; status=1; return
    ELSE
      write (gol,'("Checked ntracet in save file: okay")'); call goPr
      write (gol,'("Assuming npes is equal for both runs!")') ; call goPr
    ENDIF

    ! read level definition:
    call ReadAttribute( hdf, 'lm', lm_in, status )
    IF_NOTOK_RETURN(status=1)
    allocate( at_in(0:lm_in) )
    allocate( bt_in(0:lm_in) )

    call ReadAttribute( hdf, 'at', at_in, status )
    IF_NOTOK_RETURN(status=1)
    call ReadAttribute( hdf, 'bt', bt_in, status )
    IF_NOTOK_RETURN(status=1)

    call Init( levi_in, lm_in, at_in, bt_in, status )
    IF_NOTOK_RETURN(status=1)
    deallocate( at_in )
    deallocate( bt_in )   
    
    ! read grid definition:
    call ReadAttribute( hdf, 'im', im_in, status )
    IF_NOTOK_RETURN(status=1)
    call ReadAttribute( hdf, 'jm', jm_in, status )
    IF_NOTOK_RETURN(status=1)
    call ReadAttribute( hdf, 'xbeg', xbeg_in, status )
    IF_NOTOK_RETURN(status=1)
    call ReadAttribute( hdf, 'dx', dx_in, status )
    IF_NOTOK_RETURN(status=1)
    call ReadAttribute( hdf, 'ybeg', ybeg_in, status )
    IF_NOTOK_RETURN(status=1)
    call ReadAttribute( hdf, 'dy', dy_in, status )
    IF_NOTOK_RETURN(status=1)

    call Init( lli_in, xbeg_in+0.5*dx_in, dx_in, im_in, &
                       ybeg_in+0.5*dy_in, dy_in, jm_in, status )
    IF_NOTOK_RETURN(status=1)
    
    ! temporary storage:
    allocate( rm_in(im_in,jm_in,lm_in,1:ntracetloc) )
    
    ! open tracer masses:
    call Init( sds_rm, hdf, 'rm', status )  
    IF_NOTOK_RETURN(status=1)
    
    ! read 4D tracer field: 
    call ReadData( sds_rm, rm_in, status )  ! kg
    IF_NOTOK_RETURN(status=1)
    
    do n=1,ntracetloc    
        ! convert grids and levels:
        call Fill3D( lli(region), levi, 'n', sp(1:imr,1:jmr,1), rm(1:imr,1:jmr,1:lmr,n), &
                     lli_in, levi_in, rm_in(:,:,:,n), 'sum', status )
        IF_NOTOK_RETURN(status=1)

        ! info
        write (gol,*) 'initialised tracer ', n, names(n), '; max mass, max vmr : ', &
                           maxval(rm(1:imr,1:jmr,1:lmr,n)), &
                           maxval(rm(1:imr,1:jmr,1:lmr,n)/m(1:imr,1:jmr,1:lmr)*fscale(n)); call goPr
    end do     
    
    ! clear
    deallocate( rm_in )
    
    ! done with grid definitions:
    call Done( lli_in, status ) 
    IF_NOTOK_RETURN(status=1)
    call Done( levi_in, status ) 
    IF_NOTOK_RETURN(status=1)
    
    ! close data sets:
    call Done( sds_rm, status )  
    IF_NOTOK_RETURN(status=1)
    
    ! close hdf file (all pe)
    call Done( hdf, status )
    IF_NOTOK_RETURN(status=1)
    
    ! wait ...
    call Par_Barrier
    
    ! ok
    status = 0
    
  end subroutine ReadTracer_saved
    



end module cy2_enkf_save
