! Time-stamp: <fe1.fsl.noaa.gov:/home/jacobson/TM5/proj/ctemis/trunk/src/emission_co2_bio.F90: 21 Sep 2011 (Wed) 01:03:57 GMT>

! todo:

#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
#include "tm5.inc"

! Compute terrestrial biosphere emissions of CO2 in mol m-2 s-1.  Positive is source to atmosphere.

module emission_co2_bio
  
  use GO,		only : gol, goErr, goPr, goLabel
  use Dims,		only : nregions, nlon360,nlat180
  use Meteo,		only : t2m_dat,ssr_dat
  use Meteo,		only : Set
  use emission_common,	only : enkf_parms, field_1x1, EmisInputDir, co2_bio
  use emission_common,	only : nmembersloc

  implicit none

  character(len=*), parameter	:: mname = 'emission_co2_bio'

  private

  public	:: emission_co2_bio_init, emission_co2_bio_done, emission_co2_bio_calc

  ! use this to potentially override the EmisInputDir
  character(len=1024)	:: BioInputDir

contains

  subroutine emission_co2_bio_init(status)

    use GO,		only : TrcFile, Init, Done, ReadRc
    use global_data,	only : rcfile
    use dims,		only : im, jm, nregions, iglbsfc

    implicit none

    character(len=*), parameter	:: rname = mname//'/emission_co2_bio_init'

    integer			:: region, imr, jmr, n
    integer, intent(out)	:: status
    type(TrcFile)		:: rcF

    call Init( rcF, rcfile, status )
    IF_NOTOK_RETURN(status=1)

    call ReadRc( rcF, 'bio.input.dir', BioInputDir ,status ,default=EmisInputDir)
    IF_ERROR_RETURN(status=1)

    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)

    call Set(   t2m_dat(iglbsfc), status, used=.true. )
    IF_NOTOK_RETURN(status=1)

    call Set(   ssr_dat(iglbsfc), status, used=.true. )
    IF_NOTOK_RETURN(status=1)

    write (gol,'("[emission_co2_bio_init]	terrestrial bio input files to be read from """,a,""".")') trim(BioInputDir); call goPr

  end subroutine emission_co2_bio_init

  subroutine emission_co2_bio_calc(parms,co2_bio,flux_updated,status)

    use dims,			only : nlon360, nlat180, im,jm, sec_month, newsrun, idate, itau, staggered, nread, newmonth, iglbsfc, sec_year
    use ReadFromFile_MDF,	only : ReadFromFile
    use MDF,			only : MDF_HDF4
    use emission_data,		only : chardate, flux_to_gridbox_per_month
    use Meteo,			only : Set
    use chem_param,		only : xmc
    use go_string,		only : goNum2str

    implicit none
    
    character(len=*), parameter			:: rname = mname//'/emission_co2_bio_calc'
    
    type(enkf_parms), dimension(:), intent(in)	:: parms
    type(field_1x1), dimension(:), intent(out)	:: co2_bio
    integer, intent(out)			:: status
    logical, intent(out)			:: flux_updated

    integer					:: region, imr, jmr, imember
    real,dimension(nlon360,nlat180)		:: p0,r0
    real,pointer,dimension(:,:)			:: t2m,ssr
    character(len=10)				:: cdate
    real					:: month2year

    flux_updated = .false.

    if(mod(itau,nread) /= 0) return  !	only every nread hours

    flux_updated = .true.

    if(size(parms) .ne. nmembersloc) then
       write (gol,'("in ",a, ": received ",i," parameters but was expecting",i,".")') rname,size(parms),nmembersloc
       status = 1
       IF_NOTOK_RETURN(status=1)
    end if

    if(newmonth .or. newsrun) then
       cdate=trim(goNum2str(idate(1),'(i4.4)'))//trim(goNum2str(idate(2),'(i2.2)'))
!      TODO:  are date attributes required?       
!       call ReadFromFile(trim(BioInputDir)//'/biofireparams_'//trim(cdate)//'.hdf','rparam1',r0,status, sds_date=(/idate(1),idate(2),1,0,0,0/))
       call ReadFromFile(fname=trim(BioInputDir)//'/biofireparams_'//trim(cdate)//'.hdf',ftype=MDF_HDF4,vname='rparam1',buffer=r0,status=status)
       IF_NOTOK_RETURN(status=1)
!      TODO:  are date attributes required?       
!       call ReadFromFile(trim(BioInputDir)//'/biofireparams_'//trim(cdate)//'.hdf','pparam1',p0,status, sds_date=(/idate(1),idate(2),1,0,0,0/))
       call ReadFromFile(fname=trim(BioInputDir)//'/biofireparams_'//trim(cdate)//'.hdf',ftype=MDF_HDF4,vname='pparam1',buffer=p0,status=status)
       IF_NOTOK_RETURN(status=1)
    endif

    t2m => t2m_dat(iglbsfc)%data(:,:,1)
    ssr => ssr_dat(iglbsfc)%data(:,:,1)


    month2year = sec_year/sec_month

    do imember = 1,nmembersloc ! forward has nmembersloc = 1
       co2_bio(imember)%surf = r0*(1.5**((t2m-273.15)/10.0))  ! Q10 relation for respiration
       co2_bio(imember)%surf = co2_bio(imember)%surf+p0*ssr        ! radiation scaling for GPP
       ! co2_bio now in mol C m-2 s-1
       co2_bio(imember)%surf = co2_bio(imember)%surf*parms(imember)%surf ! apply parameters

       co2_bio(imember)%surf = co2_bio(imember)%surf * xmc/1.0e3 ! convert from mol m-2 s-1 to kgC m-2 s-1

       call flux_to_gridbox_per_month(co2_bio(imember)%surf) ! convert from kgC m-2 s-1 to kgC 1x1_gridbox-1 month-1

       write (gol,'("[emission_co2_bio_calc]	member ",i4,": terrestrial biosphere emission global integral ",f12.4," PgC/yr.")') imember,sum(co2_bio(imember)%surf)*1.0e-12*month2year
       call goPr

    end do

    nullify(t2m)
    nullify(ssr)

  end subroutine emission_co2_bio_calc

  subroutine emission_co2_bio_done(status)

    use dims,		only : nregions

    implicit none

    integer, intent(out)			:: status

    character(len=*), parameter	:: rname = mname//'/emission_co2_bio_done'

    status=0
    return

  end subroutine emission_co2_bio_done

end module emission_co2_bio
